@@ -1,7 +1,7 @@
 #@ (S-)Sym(bolic)Obj(ect) - easy creation of symbol tables and objects.
 package SymObj;
 require 5.008_001;
-our $VERSION = '0.6.1';
+our $VERSION = '0.8.0';
 our $COPYRIGHT =<<__EOT__;
 Copyright (c) 2010 - 2012 Steffen "Daode" Nurpmeso <sdaoden\@users.sf.net>.
 All rights reserved under the terms of the ISC license.
@@ -26,21 +26,18 @@ no strict 'refs'; # We fool around with that by definition, so this
 sub NONE()        { 0 }
 sub DEBUG()       { 1<<0 }
 sub VERBOSE()     { 1<<1 }
+sub DEEP_CLONE()  { 1<<2 }
 sub _USRMASK()    { 0x7 }
 
-sub _HAS_ARRAY()  { 1<<3 }
-sub _HAS_HASH()   { 1<<4 }
-sub _HAS_ALL()    { _HAS_ARRAY | _HAS_HASH }
-
 sub _CLEANHIER()  { 1<<5 }
 
-sub _UUID         { 'S-SymObj::1C8288D6-9EDA-4ECD-927F-2144B94186AD'; }
+sub _UUID         { 'S_SymObj__1C8288D6_9EDA_4ECD_927F_2144B94186AD' }
 
 our $MsgFH = *STDERR;
 our $Debug = 1; # 0,1,2
 
 sub pack_exists {
-   %{"${_[0]}::"};
+   %{"${_[0]}::"}
 }
 
 sub sym_dump {
@@ -49,13 +46,13 @@ sub sym_dump {
    if (my $i = ref $pkg) { $pkg = $i; }
    foreach (keys %{*{"${pkg}::"}}) { print $MsgFH "$_ "; }
    #while (my ($k, $v) = each %{*{"${pkg}::"}}) { print $MsgFH "$k($v) "; }
-   print $MsgFH "\n";
+   print $MsgFH "\n"
 }
 
 sub obj_dump {
    use Data::Dumper;
    my $self = shift;
-   print $MsgFH "SymObj::obj_dump(): ", Dumper($self), "\n";
+   print $MsgFH "SymObj::obj_dump(): ", Dumper($self), "\n"
 }
 
 sub sym_create { # {{{
@@ -63,7 +60,7 @@ sub sym_create { # {{{
    $flags &= _USRMASK;
    $flags |= DEBUG if $flags & VERBOSE;
    $flags |= $Debug > 1 ? (DEBUG | VERBOSE) : DEBUG if $Debug;
-   my ($pkg, $i, $j, @isa, @arri, %actorargs, @ctorovers) =
+   my ($pkg, $i, $j, @isa, %actorargs, @ctorovers) =
       (scalar caller, $flags & VERBOSE);
    print $MsgFH "SymObj::sym_create(): $pkg\n" if $i;
 
@@ -78,82 +75,8 @@ sub sym_create { # {{{
    print $MsgFH ".. (inherited VERBOSE:) SymObj::sym_create(): $pkg\n"
       if ($flags & VERBOSE) && ! $i;
 
-   # Accessor and $tfields handling {{{
-   # We use shared per-object field handler subs to minimize code blow a bit.
-   # Even that is overkill unless the %FIELDS really require that though.
-   @arri = keys %$tfields;
-   foreach $i (@arri) {
-      $j = ref $tfields->{$i};
-      if ($i =~ /^@/ || $j eq 'ARRAY') {
-         $flags |= _HAS_ARRAY;
-         last if ($flags & _HAS_ALL) == _HAS_ALL;
-      } elsif ($i =~ /^%/ || $j eq 'HASH') {
-         $flags |= _HAS_HASH;
-         last if ($flags & _HAS_ALL) == _HAS_ALL;
-      }
-   }
-
-   if ($flags & _HAS_ARRAY) {
-      print $MsgFH "\t..adding shared array handler\n" if $flags & VERBOSE;
-      *{"${pkg}::_SymObj_ArraySet"} = sub {
-         my ($self, $pub, $datum) = (shift, shift, shift);
-         my $dref = $self->{$datum};
-         $dref = $self->{$datum} = [] unless defined $dref;
-
-         foreach my $arg (@_) {
-            if (ref $arg eq 'ARRAY') {
-               push(@$dref, $_) foreach (@$arg);
-            } elsif (ref $arg eq 'HASH') {
-               while (my ($k, $v) = each %$arg) {
-                  push @$dref, $k;
-                  push @$dref, $v;
-               }
-            } else {
-               push @$dref, $arg;
-            }
-         }
-         $dref;
-      };
-   }
-
-   if ($flags & _HAS_HASH) {
-      print $MsgFH "\t..adding shared hash handler\n" if $flags & VERBOSE;
-      *{"${pkg}::_SymObj_HashSet"} = sub {
-         my ($self, $pub, $datum) = (shift, shift, shift);
-         my $dref = $self->{$datum};
-         $dref = $self->{$datum} = {} unless defined $dref;
-
-         my $k = undef;
-         foreach my $arg (@_) {
-            if (defined $k) {
-               $dref->{$k} = $arg;
-               $k = undef;
-               next;
-            }
-            if (ref $arg eq 'HASH') {
-               while (my ($k, $v) = each %$arg) {
-                  $dref->{$k} = $v;
-               }
-            } elsif (ref $arg eq 'ARRAY') {
-               while (@$arg > 1) {
-                  my $v = pop @$arg;
-                  my $k = pop @$arg;
-                  $dref->{$k} = $v;
-               }
-               print $MsgFH "! ${pkg}::$pub(): wrong array member count!\n"
-                  if @$arg != 0 && ($flags & DEBUG);
-            } else {
-               $k = $arg;
-            }
-         }
-         print $MsgFH "! ${pkg}::$pub(): '$k' key without a value\n"
-            if defined $k && ($flags & DEBUG);
-         $dref;
-      };
-   }
-
    # Create accessor symtable entries
-   foreach $j (@arri) {
+   foreach $j (keys %$tfields) {
       sub TYPE_ARRAY()     { 1<<0 }
       sub TYPE_HASH()      { 1<<1 }
       sub TYPE_EXCLUDE()   { 1<<2 }
@@ -163,7 +86,7 @@ sub sym_create { # {{{
       $j =~ /^([@%])?([?!])?(_)?(.*)/;
       $tj |= $1 eq '@' ? TYPE_ARRAY : TYPE_HASH if defined $1;
       $tj |= $2 eq '?' ? TYPE_EXCLUDE : TYPE_RDONLY if defined $2;
-      print $MsgFH "\tSymbol '$pj': does NOT start with underscore _!\n"
+      print $MsgFH "\tSymbol '$pj': does NOT start with _!  THIS WILL FAIL!!\n"
          if ! defined $3 && ($flags & DEBUG);
       unless (defined $4 && length $4) {
          print $MsgFH "\tSymbol '$pj': consists only of modifier and/or ",
@@ -196,13 +119,13 @@ sub sym_create { # {{{
       }
       $actorargs{$pj} = \$tfields->{$xj};
 
-      # Hope that perl(1) optimizes away the TYPE_* conditions..
+      # (And hope that perl(1) optimizes away some closure conditions..)
       if (($tj & TYPE_ARRAY) || $i eq 'ARRAY') {
          print $MsgFH "\tsub $pj: array-based\n" if $flags & VERBOSE;
-         *{"${pkg}::__$pj"} = sub { $_[0]->{$xj}; };
+         *{"${pkg}::__$pj"} = sub { $_[0]->{$xj} };
          *{"${pkg}::$pj"} = sub {
             my $self = $_[0];
-            if (($self = ref $self) ne '' && %{"${self}::"}) {
+            if (($self = ref $self) && %{"${self}::"}) {
                $self = shift;
                if ($tj & TYPE_EXCLUDE) {
                   SymObj::_complain_exclude($pkg, $pj) if $flags & DEBUG;
@@ -215,16 +138,16 @@ sub sym_create { # {{{
             if (! defined $f || @_) {
                SymObj::_complain_rdonly($pkg, $pj)
                   if ($tj & TYPE_RDONLY) && @_ && ($flags & DEBUG);
-               $f = "${pkg}::_SymObj_ArraySet"->($self, $pj, $xj, @_);
+               $f = SymObj::_array_set($pkg, $self, $pj, $xj, @_);
             }
-            wantarray ? @$f : $f;
+            wantarray ? @$f : $f
          };
       } elsif (($tj & TYPE_HASH) || $i eq 'HASH') {
          print $MsgFH "\tsub $pj: hash-based\n" if $flags & VERBOSE;
-         *{"${pkg}::__$pj"} = sub { $_[0]->{$xj}; };
+         *{"${pkg}::__$pj"} = sub { $_[0]->{$xj} };
          *{"${pkg}::$pj"} = sub {
             my $self = $_[0];
-            if (($self = ref $self) ne '' && %{"${self}::"}) {
+            if (($self = ref $self) && %{"${self}::"}) {
                $self = shift;
                if ($tj & TYPE_EXCLUDE) {
                   SymObj::_complain_exclude($pkg, $pj) if $flags & DEBUG;
@@ -237,18 +160,18 @@ sub sym_create { # {{{
             if (! defined $f || @_) {
                SymObj::_complain_rdonly($pkg, $pj)
                   if ($tj & TYPE_RDONLY) && @_ && ($flags & DEBUG);
-               $f = &{${"${pkg}::"}{_SymObj_HashSet}}($self, $pj, $xj, @_);
+               $f = SymObj::_hash_set($pkg, $self, $pj, $xj, @_);
             }
-            wantarray ? %$f : $f;
+            wantarray ? %$f : $f
          };
       } else {
          # Scalar (or "typeless")
          print $MsgFH "\tsub $pj: scalar-based ('untyped')\n"
             if $flags & VERBOSE;
-         *{"${pkg}::__$pj"} = sub { \$_[0]->{$xj}; };
+         *{"${pkg}::__$pj"} = sub { \$_[0]->{$xj} };
          *{"${pkg}::$pj"} = sub {
             my $self = $_[0];
-            if (($self = ref $self) ne '' && %{"${self}::"}) {
+            if (($self = ref $self) && %{"${self}::"}) {
                $self = shift;
                if ($tj & TYPE_EXCLUDE) {
                   SymObj::_complain_exclude($pkg, $pj) if $flags & DEBUG;
@@ -262,11 +185,10 @@ sub sym_create { # {{{
                   if ($tj & TYPE_RDONLY) && ($flags & DEBUG);
                $self->{$xj} = shift;
             }
-            $self->{$xj};
+            $self->{$xj}
          };
       }
    }
-   # }}}
 
    # Variable fields
    ${"${pkg}::"}{_SymObj_PACKAGE} = $pkg;
@@ -280,19 +202,185 @@ sub sym_create { # {{{
    if (! defined $ctor || ref $ctor ne 'CODE') {
       my $_c = $ctor;
       $_c = '__ctor' unless defined $_c;
-      $ctor = sub { SymObj::_find_usr_ctor(shift, $pkg, $_c); };
+      $ctor = sub { SymObj::_find_usr_ctor(shift, $pkg, $_c) };
    }
    ${"${pkg}::"}{_SymObj_USR_CTOR} = $ctor;
 
    # new()
    if ($flags & DEBUG) {
-      $ctor = sub { SymObj::_ctor_dbg($pkg, shift, \@_); };
+      $ctor = sub { SymObj::_ctor_dbg($pkg, shift, \@_) };
    } elsif ($flags & _CLEANHIER) {
-      $ctor = sub { SymObj::_ctor_cleanhier($pkg, shift, \@_); };
+      $ctor = sub {
+         my ($class, $self) = (shift, {});
+         $self = bless $self, $class;
+         # Embed arguments
+         $self = SymObj::_ctor_argembed($pkg, $class, $self, \%actorargs,
+               \%actorargs, \@_);
+         # Fill what is not yet filled from arguments..
+         my $f = $flags & DEEP_CLONE;
+         while (($i, $j) = each %actorargs) {
+            $i = '_' . $i;
+            next if exists $self->{$i};
+            $self->{$i} = SymObj::clone_ref($$j, $f);
+         }
+         # Call user CTORs in correct order..
+         foreach $pkg (@isa) {
+            if (defined(my $sym = ${"${pkg}::"}{_SymObj_USR_CTOR})) {
+               &$sym($self);
+            }
+         }
+         $self
+      };
    } else {
-      $ctor = sub { SymObj::_ctor_dirtyhier($pkg, shift, \@_); };
+      $ctor = sub { SymObj::_ctor_dirtyhier($pkg, shift, \@_) };
    }
    *{"${pkg}::new"} = $ctor;
+   1
+} # }}}
+
+sub clone_ref { # {{{
+   my ($r, $deep) = @_;
+
+   if (! $deep) {
+      if (ref $r eq 'ARRAY') {
+         my @a;
+         push @a, $_ foreach (@$r);
+         $r = \@a;
+      } elsif (ref $r eq 'HASH') {
+         my (%h, $hk, $hv);
+         while (($hk, $hv) = each %$r) { $h{$hk} = $hv; }
+         $r = \%h;
+      }
+   } else {
+      if (ref $r eq 'ARRAY') {
+         my @a;
+         push @a, SymObj::clone_ref($_, 1) foreach (@$r);
+         $r = \@a;
+      } elsif (ref $r eq 'HASH') {
+         my (%h, $hk, $hv);
+         while (($hk, $hv) = each %$r) { $h{$hk} = SymObj::clone_ref($hv, 1); }
+         $r = \%h;
+      }
+   }
+   $r
+} # }}}
+
+sub _resolve_tree { # {{{
+   my ($pkg, $_actorargs, $_p, $_f, $_isa) = @_;
+   foreach my $c (@{${"${_p}::"}{ISA}}) {
+      unless (%{"${c}::"}) {
+         print $MsgFH "${pkg}: $_p: \@ISA contains non-existent ",
+            "class '$c'!\n" if $$_f & DEBUG;
+         next;
+      }
+
+      my $j = ${"${c}::"}{_SymObj_FLAGS};
+      unless (defined $j) {
+         print $MsgFH "${pkg}: $_p:  '$c' not SymObj managed: hierarchy not ",
+            "clean, STOP!\n" if $$_f & VERBOSE;
+         $$_f &= ~_CLEANHIER;
+         next;
+      }
+      $$_f |= $j & (DEBUG | VERBOSE); # Inherit debug states
+      if (! ($j & _CLEANHIER) && ($$_f & _CLEANHIER)) {
+         $$_f &= ~_CLEANHIER;
+         print $MsgFH "${pkg}: $_p: '$c' says hierarchy is not clean..\n"
+            if $$_f & VERBOSE;
+      }
+
+      while (my ($k, $v) = each %{${"${c}::"}{_SymObj_ALL_CTOR_ARGS}}) {
+         $_actorargs->{$k} = $v;
+      }
+
+      _resolve_tree($pkg, $_actorargs, $c, $_f, $_isa)
+         if defined ${"${c}::"}{ISA};
+      push @$_isa, $c;
+   }
+   1
+} # }}}
+
+sub _complain_exclude {
+   my ($pkg, $pub) = @_;
+   print $MsgFH "${pkg}::$pub(): field can't be accessed through object, ",
+      "accessing class-static!\n";
+   1
+}
+
+sub _complain_rdonly {
+   my ($pkg, $pub) = @_;
+   print $MsgFH "${pkg}::$pub(): write access to READONLY field!\n";
+   1
+}
+
+sub _hash_set { # {{{
+   my ($pkg, $self, $pub, $datum) = (shift, shift, shift, shift);
+   my ($flags, $dref, $k) = (${"${pkg}::"}{_SymObj_FLAGS}, $self->{$datum});
+
+   $dref = $self->{$datum} = {} unless defined $dref;
+
+   foreach my $arg (@_) {
+      if (defined $k) {
+         $dref->{$k} = $arg;
+         $k = undef;
+         next;
+      }
+      if (ref $arg eq 'HASH') {
+         while (my ($k, $v) = each %$arg) {
+            $dref->{$k} = $v;
+         }
+      } elsif (ref $arg eq 'ARRAY') {
+         while (@$arg > 1) {
+            my $v = pop @$arg;
+            my $k = pop @$arg;
+            $dref->{$k} = $v;
+         }
+         print $MsgFH "! ${pkg}::$pub(): wrong array member count!\n"
+            if @$arg != 0 && ($flags & DEBUG);
+      } else {
+         $k = $arg;
+      }
+   }
+   print $MsgFH "! ${pkg}::$pub(): '$k' key without a value\n"
+      if defined $k && ($flags & DEBUG);
+   $dref
+} # }}}
+
+sub _array_set { # {{{
+   my ($pkg, $self, $pub, $datum) = (shift, shift, shift, shift);
+   my $dref = $self->{$datum};
+
+   $dref = $self->{$datum} = [] unless defined $dref;
+
+   foreach my $arg (@_) {
+      if (ref $arg eq 'ARRAY') {
+         push @$dref, $_ foreach (@$arg);
+      } elsif (ref $arg eq 'HASH') {
+         while (my ($k, $v) = each %$arg) {
+            push @$dref, $k;
+            push @$dref, $v;
+         }
+      } else {
+         push @$dref, $arg;
+      }
+   }
+   $dref
+} # }}}
+
+sub _find_usr_ctor { # {{{
+   # No constructor was given to sym_create(), or it was no code-ref.
+   # Try to find out what the user wants.
+   my ($self, $pkg, $ctor) = @_;
+   my $flags = ${"${pkg}::"}{_SymObj_FLAGS};
+   unless (defined ${"${pkg}::"}{$ctor}) {
+      print $MsgFH "${pkg}: no user ctor\n" if $flags & VERBOSE;
+      delete ${"${pkg}::"}{_SymObj_USR_CTOR};
+   } else {
+      print $MsgFH "${pkg}: resolved user ctor '$ctor'\n" if $flags & VERBOSE;
+      $ctor = ${"${pkg}::"}{$ctor};
+      ${"${pkg}::"}{_SymObj_USR_CTOR} = $ctor;
+      &$ctor($self);
+   }
+   1
 } # }}}
 
 sub _ctor_dbg { # {{{
@@ -353,58 +441,33 @@ j_OVW:}
       print $MsgFH "${pkg}: $class->new(): odd argument discarded!\n";
    }
 
-   # Use generic internal accessors here
-   while (@$argaref) {
-      $k = shift @$argaref;
-      $i = '_' . $k;
-      $j = shift @$argaref;
-      unless (exists $actorargs->{$k}) {
-         next if $init_chain;
-         print $MsgFH "${pkg}: $class->new(): unknown argument: '$k'\n";
-         next;
-      }
-      next unless exists $tfields->{$i};
-
-      if (ref $tfields->{$i} eq 'ARRAY') {
-         $self->_SymObj_ArraySet('new()', $i, $j);
-      } elsif (ref $tfields->{$i} eq 'HASH') {
-         unless (ref $j eq 'ARRAY' || ref $j eq 'HASH') {
-            print $MsgFH "${pkg}: $class->new(): ",
-               "'$k' requires ARRAY or HASH argument\n";
-            next;
-         }
-         $self->_SymObj_HashSet('new()', $i, $j);
-      } else {
-         $self->{$i} = $j;
-      }
-   }
+   # Embed arguments
+   $self->{_UID} = undef if $init_chain;
+   $self = SymObj::_ctor_argembed($pkg, $class, $self, $actorargs, $tfields,
+         $argaref);
+   delete $self->{_UID} if $init_chain;
 
    # Finally: fill in yet unset members of $self via the per-class template.
-   # By default anon-hashes and -arrays get reference-copied;
-   # we however *do* need a detached (deep) copy!
-   while (($k, $j) = each %$tfields) {
+   $j = $flags;
+   if (exists ${"${class}::"}{_SymObj_FLAGS}) {
+      $j = ${"${class}::"}{_SymObj_FLAGS};
+   }
+   $j &= DEEP_CLONE;
+   while (($k, $i) = each %$tfields) {
       next if exists $self->{$k};
-      unless (ref $j) {
-         $self->{$k} = $j;
-      } elsif (ref $j eq 'ARRAY') {
-         my @a;
-         push @a, $_ foreach (@$j);
-         $self->{$k} = \@a;
-      } elsif (ref $j eq 'HASH') {
-         my (%h, $hk, $hv);
-         while (($hk, $hv) = each %$j) { $h{$hk} = $hv; }
-         $self->{$k} = \%h;
-      } else {
+      if (ref $i && ref $i ne 'HASH' && ref $i ne 'ARRAY') {
          print $MsgFH "${pkg}: $class->new(): value of '$k' has an ",
             "unsupported type!\n";
+         next;
       }
+      $self->{$k} = SymObj::clone_ref($i, $j);
    }
 
    # Call further init code, if defined
    if (defined($i = ${"${pkg}::"}{_SymObj_USR_CTOR})) {
       &$i($self);
    }
-   $self;
+   $self
 } # }}}
 
 sub _ctor_dirtyhier { # (Stripped version of _dbg) {{{
@@ -437,157 +500,60 @@ j_OVW:}
    $self = {} unless defined $self;
    $self = bless $self, $class;
 
-   while (@$argaref) {
-      $k = shift @$argaref;
-      $i = '_' . $k;
-      $j = shift @$argaref;
-      unless (exists $actorargs->{$k}) {
-         next;
-      }
-      next unless exists $tfields->{$i};
+   # Embed arguments
+   $self = SymObj::_ctor_argembed($pkg, $class, $self, $actorargs, $tfields,
+         $argaref);
 
-      if (ref $tfields->{$i} eq 'ARRAY') {
-         $self->_SymObj_ArraySet('new()', $i, $j);
-      } elsif (ref $tfields->{$i} eq 'HASH') {
-         unless (ref $j eq 'ARRAY' || ref $j eq 'HASH') {
-            next;
-         }
-         $self->_SymObj_HashSet('new()', $i, $j);
-      } else {
-         $self->{$i} = $j;
-      }
+   # Fill what is not yet filled from arguments..
+   if (exists ${"${class}::"}{_SymObj_FLAGS}) {
+      $j = ${"${class}::"}{_SymObj_FLAGS};
+   } else {
+      $j = ${"${pkg}::"}{_SymObj_FLAGS};
    }
-
-   while (($k, $j) = each %$tfields) {
+   $j &= DEEP_CLONE;
+   while (($k, $i) = each %$tfields) {
       next if exists $self->{$k};
-      unless (ref $j) {
-         $self->{$k} = $j;
-      } elsif (ref $j eq 'ARRAY') {
-         my @a;
-         push @a, $_ foreach (@$j);
-         $self->{$k} = \@a;
-      } elsif (ref $j eq 'HASH') {
-         my (%h, $hk, $hv);
-         while (($hk, $hv) = each %$j) { $h{$hk} = $hv; }
-         $self->{$k} = \%h;
-      }
+      $self->{$k} = SymObj::clone_ref($i, $j);
    }
 
    if (defined($i = ${"${pkg}::"}{_SymObj_USR_CTOR})) {
       &$i($self);
    }
-   $self;
+   $self
 } # }}}
 
-sub _ctor_cleanhier { # {{{
-   my ($pkg, $class, $argaref) = @_;
-   my $self = {};
-   $self = bless $self, $class;
-   my $isa = ${"${pkg}::"}{_SymObj_ISA};
-   my $allargs = ${"${pkg}::"}{_SymObj_ALL_CTOR_ARGS};
+sub _ctor_argembed { # {{{
+   my ($pkg, $class, $self, $argsr, $myargsr, $usrr) = @_;
+   my ($isident, $k, $pk, $v, $r) = ($argsr == $myargsr);
 
-   # Embed arguments
-   while (@$argaref) {
-      my $k = shift @$argaref;
-      my $pk = '_' . $k;
-      my $v = shift @$argaref;
-      my $tv = ${$allargs->{$k}};
-      if (ref $tv eq 'ARRAY') {
-         $self->_SymObj_ArraySet('new()', $pk, $v);
-      } elsif (ref $tv eq 'HASH') {
-         $self->_SymObj_HashSet('new()', $pk, $v);
+   while (@$usrr) {
+      $k = shift @$usrr;
+      $v = shift @$usrr;
+      unless (exists $argsr->{$k}) {
+         next if exists $self->{_UID};
+         print $MsgFH "${pkg}: $class->new(): unknown argument: '$k'\n";
+         next;
+      }
+      $r = ${$argsr->{$k}};
+      $pk = '_' . $k;
+      # In non-optimized case: is this an arg of $pkg?
+      next unless $isident || exists $myargsr->{$pk};
+
+      if (ref $r eq 'ARRAY') {
+         Symobj::_array_set($pkg, $self, $k, $pk, $v);
+      } elsif (ref $r eq 'HASH') {
+         unless (ref $v eq 'ARRAY' || ref $v eq 'HASH') {
+            print $MsgFH "${pkg}: $class->new(): ",
+               "'$k' requires ARRAY or HASH argument\n";
+            next;
+         }
+         Symobj::_hash_set($pkg, $self, $k, $pk, $v);
       } else {
          $self->{$pk} = $v;
       }
    }
-
-   # Fill what is not yet filled from arguments..
-   while (my ($k, $v) = each %$allargs) {
-      $k = '_' . $k;
-      next if exists $self->{$k};
-      $v = $$v;
-      if (ref $v eq 'ARRAY') {
-         my @a;
-         push @a, $_ foreach (@$v);
-         $self->{$k} = \@a;
-      } elsif (ref $v eq 'HASH') {
-         my (%h, $hk, $hv);
-         while (($hk, $hv) = each %$v) { $h{$hk} = $hv; }
-         $self->{$k} = \%h;
-      } else {
-         $self->{$k} = $v;
-      }
-   }
-
-   # Call user CTORs in correct order..
-   foreach $pkg (@$isa) {
-      if (defined(my $sym = ${"${pkg}::"}{_SymObj_USR_CTOR})) {
-         &$sym($self);
-      }
-   }
-   $self;
+   $self
 } # }}}
-
-sub _resolve_tree { # {{{
-   my ($pkg, $_actorargs, $_p, $_f, $_isa) = @_;
-   foreach my $c (@{${"${_p}::"}{ISA}}) {
-      unless (%{"${c}::"}) {
-         print $MsgFH "${pkg}: $_p: \@ISA contains non-existent ",
-            "class '$c'!\n" if $$_f & DEBUG;
-         next;
-      }
-
-      my $j = ${"${c}::"}{_SymObj_FLAGS};
-      unless (defined $j) {
-         print $MsgFH "${pkg}: $_p:  '$c' not SymObj managed: hierarchy not ",
-            "clean, STOP!\n" if $$_f & VERBOSE;
-         $$_f &= ~_CLEANHIER;
-         next;
-      }
-      $$_f |= $j & (DEBUG | VERBOSE); # Inherit debug states
-      if (! ($j & _CLEANHIER) && ($$_f & _CLEANHIER)) {
-         $$_f &= ~_CLEANHIER;
-         print $MsgFH "${pkg}: $_p: '$c' says hierarchy is not clean..\n"
-            if $$_f & VERBOSE;
-      }
-
-      while (my ($k, $v) = each %{${"${c}::"}{_SymObj_ALL_CTOR_ARGS}}) {
-         $_actorargs->{$k} = $v;
-      }
-
-      _resolve_tree($pkg, $_actorargs, $c, $_f, $_isa)
-         if defined ${"${c}::"}{ISA};
-      push @$_isa, $c;
-   }
-} # }}}
-
-sub _complain_exclude {
-   my ($pkg, $pub) = @_;
-   print $MsgFH "${pkg}::$pub(): field can't be accessed through object, ",
-      "accessing class-static!\n";
-}
-
-sub _complain_rdonly {
-   my ($pkg, $pub) = @_;
-   print $MsgFH "${pkg}::$pub(): write access to READONLY field!\n";
-}
-
-sub _find_usr_ctor { # {{{
-   # No constructor was given to sym_create(), or it was no code-ref.
-   # Try to find out what the user wants.
-   my ($self, $pkg, $ctor) = @_;
-   my $flags = ${"${pkg}::"}{_SymObj_FLAGS};
-   unless (defined ${"${pkg}::"}{$ctor}) {
-      print $MsgFH "${pkg}: no user ctor\n" if $flags & VERBOSE;
-      delete ${"${pkg}::"}{_SymObj_USR_CTOR};
-   } else {
-      print $MsgFH "${pkg}: resolved user ctor '$ctor'\n" if $flags & VERBOSE;
-      $ctor = ${"${pkg}::"}{$ctor};
-      ${"${pkg}::"}{_SymObj_USR_CTOR} = $ctor;
-      &$ctor($self);
-   }
-} # }}}
-
 1;
 __END__
 # POD {{{
@@ -598,17 +564,19 @@ S-SymObj -- an easy way to create symbol-tables and objects.
 
 =head1 SYNOPSIS
 
-   # You need to require it in a BEGIN{}..; try out $Debug= 1/2
-   BEGIN { require SymObj; $SymObj::Debug = 0; }
+   use diagnostics -verbose;
+   use strict;
+   use warnings;
+   # You need to require it in a BEGIN{}..; $Debug may be one of 0/1/2
+   BEGIN { require SymObj; $SymObj::Debug = 2; }
 
    # Accessor subs return references for hashes and arrays (but shallow
-   # copy in wantarray context) scalars are returned "as-is"
+   # clones in wantarray context), scalars are returned "as-is"
    {package X1;
-      BEGIN {
-         SymObj::sym_create(SymObj::NONE, { # (NONE is 0..)
-               _name => '', _array => [qw(Is Easy)],
-               _hash => {To => 'hv1', Use => 'hv2'} });
-      }
+      SymObj::sym_create(SymObj::NONE, { # (NONE is 0..)
+         _name => '', _array => [qw(Is Easy)],
+         _hash => {To => 'hv1', Use => 'hv2'},
+         boing => undef }); # <- $SymObj::Debug will complain!  FAILS!
    }
    my $o = X1->new(name => 'SymObj');
    print $o->name, ' ';
@@ -617,19 +585,23 @@ S-SymObj -- an easy way to create symbol-tables and objects.
 
    # Unknown arguments are detected when DEBUG/VERBOSE is enabled.
    {package X2;
-      our (@ISA); BEGIN { @ISA = ('X1');
-         SymObj::sym_create(0, {}); # <- adds no fields on its own
-      }
+      our @ISA = ('X1');
+      SymObj::sym_create(0, {}); # <- adds no fields on its own
+   }
+   # (Clean hierarchy has optimized constructor which is used, then)
+   if ($SymObj::Debug != 0) {
+      $o = X2->new(name => 'It detects some misuses (if $Debug > 0)',
+         'un' => 'known argument catched');
+   } else {
+      $o = X2->new(name => 'It detects some misuses (if $Debug > 0)');
    }
-   my $o = X2->new(name => 'It detects some misuses', 'un' => 'known');
    print $o->name, "\n";
 
    # Fields which mirror fieldnames of superclasses define overrides.
    {package X3;
-      our (@ISA); BEGIN { @ISA = ('X2');
-         SymObj::sym_create(0, { '_name' => 'Auto superclass-ovw'},
-            sub { my $self = shift; print "X3 usr ctor\n"; });
-      }
+      our @ISA = ('X2');
+      SymObj::sym_create(0, { '_name' => 'Auto superclass-ovw'},
+         sub { my $self = shift; print "X3 usr ctor\n"; });
    }
    $o = X3->new();
    print $o->name, "\n";
@@ -638,9 +610,8 @@ S-SymObj -- an easy way to create symbol-tables and objects.
    # values by using the @/% type modifiers; the objects themselves
    # are lazy-created as necessary, then...
    {package X4;
-      our (@ISA); BEGIN { @ISA = ('X3');
-         SymObj::sym_create(0, { '%_hash2'=>undef, '@_array2'=>undef });
-      }
+      our @ISA = ('X3');
+      SymObj::sym_create(0, { '%_hash2'=>undef, '@_array2'=>undef });
       sub __ctor { my $self = shift; print "X4 usr ctor\n"; }
    }
    $o = X4->new(name => 'A X4');
@@ -662,19 +633,18 @@ S-SymObj -- an easy way to create symbol-tables and objects.
 
 SymObj.pm provides an easy way to create and construct symbol-tables
 and objects.  With a simple hash one defines the fields an object
-should have.  An automatically instantiated constructor can then be
-used to create the object, and the generated accessor subs implement
-a I<feed in and forget> approach when they are about to manage arrays
-or hashes, trying to handle all kinds of arguments; this is also true
-for the constructor.
+should have, and the desired accessors and a constructor subroutine
+will be automatically generated.  Subroutines which deal with arrays
+and hashes implement a I<feed in and forget> approach, in that hash
+and array arguments are unfolded; the constructor will behave this way
+for all managed array and hash fields.
 
-If debug was enabled upon creation time a constructor which does a lot
-of argument checking and more is used, which is pretty useful in times
-when the interface is unstable.  Otherwise a different constructor is
-used which implements no checking at all; and if the object in question
-is the head of a "clean" object tree, one that is entirely managed by
-S-SymObj, then indeed a super-fast super-lean constructor implementation
-is used that'll rock your house.
+If debug was enabled upon symbol-table creation time a verbose and
+error checking constructor is used.  Otherwise a straight constructor
+without any checking will be created; and if the managed object is the
+head of a "clean" object tree, i.e., one that is entirely managed by
+S-SymObj, then a super-fast super-lean constructor implementation is
+used that'll rock your house.
 
 SymObj.pm works for Multiple-Inheritance as good as perl(1) allows.
 (That is to say that only one straight object tree can be used, further
@@ -693,7 +663,7 @@ S-SymObj managed and non-managed classes in B<mixed order>, as in
 I<MANAGED subclassof NON-MANAGED subclassof MANAGED>, because tree
 traversal actually stops once a I<NON-MANAGED> class is seen.  This
 is logical, because non-managed classes do not contain the necessary
-information for S-SymObj.
+information for S-SymObj to function.
 
 The SymObj module is available on CPAN.  The S-SymObj project is located
 at L<http://sdaoden.users.sourceforge.net/code.html>.  It is developed
@@ -705,7 +675,7 @@ L<http://sourceforge.net/p/ssymobj/code/>).
 
 =over
 
-=item C<$VERSION> (string, i.e., '0.6.1')
+=item C<$VERSION> (string, i.e., '0.8.0')
 
 A version string.
 
@@ -721,14 +691,14 @@ written to.
 
 =item C<$Debug> (0=off, 1=on, 2=verbose; default: 1)
 
-If set to a value different to 0 then a lot of debug checks are
+If set to a value different than 0 then a lot of debug checks are
 performed, and a rather slow object-constructor path is chosen (see
 below).  If set to a value greater than 1 then message verbosity is
 increased.  All messages go to L<"MsgFH">.
 
 B<Note:> changing this value later on will neither affect the
-object-constructor paths nor the per-object settings of all classes
-and class-objects that have already been created/instantiated.
+object-constructor paths nor the per-object settings of classes and
+class-objects that already have been instantiated.
 
 =item C<NONE>
 
@@ -746,6 +716,38 @@ Bit-flag for L<"sym_create">, meaning to enable verbosity on a per-object
 level.  Cannot be used to overwrite an enabled L<"Debug">, but will
 be recognized otherwise.
 
+=item C<DEEP_CLONE>
+
+Bit-flag for L<"sym_create">.  If this is set then values from the
+per-class reference hash are deep cloned upon object construction time.
+Normally these fields are reference-copied except for the first,
+immediate level, which is shallow cloned; i.e., if the value is an
+array or hash, that very array or hash is cloned, but the values within
+it are only reference copied.  But if this flag is set then array and
+hash members are I<recursively> cloned in addition.  E.g.:
+
+   SymObj::sym_create(SymObj::NONE, { array => [1, [2, 3]] });
+   ->
+   $o = XXX->new;
+   die unless $o->array->[1]->[0] == 2;
+   XXX::array()->[0] = -1;
+   die unless $o->array->[0] == 1;
+   XXX::array()->[1]->[0] = -2;
+   die unless $o->array->[1]->[0] == -2;
+
+   SymObj::sym_create(SymObj::DEEP_CLONE, { array => [1, [2, 3]] });
+   ->
+   $o = XXX->new;
+   die unless $o->array->[1]->[0] == 2;
+   XXX::array()->[0] = -1;
+   die unless $o->array->[0] == 1;
+   XXX::array()->[1]->[0] = -2;
+   die unless $o->array->[1]->[0] == 2;
+
+This flag will be overwritten by subclasses, i.e., the C<DEEP_CLONE>
+state of the I<actually instantiated> subclass is what is used to
+decide wether deep cloning shall be performed or not.
+
 =item C<pack_exists(C<$1>=string=package/class)>
 
 Check wether the class (package) $1 exists.
@@ -770,9 +772,9 @@ L<"$_SymObj_FIELDS"> and used for the time being!  It should thus be
 assumed that ownership of C<$2> is overtaken by S-SymObj.
 
 C<$1> can be used to set per-class (package that is) flags, like
-L<"DEBUG"> or L<"VERBOSE">.  Flags set like that will be inherited by
-all subclasses (unless otherwise noted).  It is not possible to lower
-the global L<"Debug"> state on this basis.
+L<"DEBUG"> or L<"DEEP_CLONE">.  Flags set like that will be inherited
+by all subclasses (unless otherwise noted).  It is not possible to
+lower the global L<"Debug"> state on this basis, though.
 
 C<$3> is the optional per-object user constructor, which can be used
 to perform additional setup of C<$self> as necessary.  These user
@@ -793,7 +795,8 @@ If in doubt, pass a code-reference.
 SymObj generally "enforces" privacy (by definition) via an underscore
 prefix: all keys of C<$2> are expected to start with an underscore,
 but the public accessor methods will miss that (C<_data> becomes
-C<data>).
+C<data>).  This is actually a requirement that is checked if debug is
+enabled, since SymObj won't function properly for this field otherwise.
 
 The created accessor subs work as methods if the first argument that
 is seen is a reference that seems to be a class instantiation (i.e.,
@@ -802,17 +805,17 @@ C<$self>, as in C<$self-E<gt>name()>) and as functions otherwise
 hash (C<$2>) is used!  (Note that no locking is performed in the latter
 case, i.e., this should not be done in multithreaded programs.)  If
 they act upon arrays or hashes they'll return references to the members
-by default, but do return copies in C<wantarray> context instead.
+by default, but do return shallow clones in C<wantarray> context instead.
 
 If a key in C<$2> is prefixed with an AT or a PERCENT, as in C<'@_name'>
 or C<'%_name'>, respectively, then the field in question is assumed
 to be an array or hash, respectively.  By default S-SymObj uses the
 value to figure out which kind of accessor has to be used, but for
-that the value must be set to a value different than C<undef>, which
-is sometimes not desirable, e.g., when a field should be lazy allocated,
+that the value must be set to a value different to C<undef>, which is
+sometimes not desirable, e.g., when a field should be lazy allocated,
 only if it is really used.  Note that the generic accessors will
 automatically instantiate an empty array/hash as necessary in these
-cases.
+cases once the field is accessed first.
 
 After the (optional) C<@> or C<%> type modifier, one may use (also
 optionally) C<?> or C<!>, mutually exclusive, as an access modifier.
@@ -825,29 +828,38 @@ this case.
 Whatever type and/or access modifier(s) was/were present, they will
 be stripped from the field name, just like a following underscore will,
 i.e., a field C<'@!_name'> will actually end up as C<_name> in the
-class-static hash, with an accessor sub named C<name>.
+class-static hash, with the accessor subroutinge C<name>.
 
 In addition to those accessor subs there will I<always> be private
 accessor subs be created which use the public name prefixed with two
 underscores, as in C<$self-E<gt>__name()>.  These subs do nothing
 except returning a reference to the field in question.  They're ment
-to be used instead of direct access of members in some contexts, i.e.,
+to be used instead of direct access to members in some contexts, i.e.,
 for encapsulation purposes.  Note that they do I<not> automatically
 instantiate lazy allocated fields.
 
 Note that, if any of the superclasses of C<$1>, as detected through
 its C<@ISA> array, provides fields which have identical names to what
-is provided in C<$2>, this situation is treated as an overwrite request,
-and it is verified that the value type matches.  Unfortunately the
-subclass will create accessor subs on its own, because users need to
-be able to adjust the class-static data.  And, also unfortunately,
-different access policies won't be detected.
+is provided in C<$2>, one specifies an overwrite request, and it is
+verified that the value type matches.  Unfortunately the subclass will
+create accessor subs on its own, because users need to be able to
+adjust the class-static data.  And, also unfortunately, different
+access policies won't be detected.
+
+=item C<clone_ref($1=ref, $2=boolean=deep-clone-reference)>
+
+Calling this support sub clones the variable C<$1>, which is treated
+as a plain scalar unless it is a HASH or ARRAY reference.  If C<$2>
+is not false then C<clone_ref()> recurses for the variable, cloning
+all levels until no more HASH or ARRAY references remain (uncloned).
+The final perl variable is returned.  (This is the logic behind
+C<DEEP_CLONE>.)
 
 =back
 
 =head1 INJECTED SYMBOLS
 
-For completeness, here is a list of all the symbols that S-SymObj
+For completeness sake, here is a list of all the symbols that S-SymObj
 creates internally for each managed package.
 
 =over
@@ -890,13 +902,17 @@ resolved).
 
 The auto-generated public class constructor.
 
-=item C<_SymObj_ArraySet()>
+=back
 
-Shared array handler, only if needed.
+=head1 NEWS
 
-=item C<_SymObj_HashSet()>
+=over
 
-Shared hash handler, only if needed.
+=item v0.8.0
+
+Support for L<"DEEP_CLONE"> was added.
+The code has been reordered.
+Much lesser generated code is injected into classes.
 
 =back
 
@@ -913,7 +929,7 @@ initialize default objects if none has been given by user.
 
 Finally: realize that perl(1) ships with struct and class and similar
 things which head in the very same direction as S-SymObj.  I want to
-point out that i wrote this package the hard way.  It is me.
+point out that i wrote this package the hard way.  It is I.
 
 =head1 LICENSE
 
